use crate::level::{Level, LevelElement};
use crate::player::Player;

pub struct Game {
    pub player: Player,
    pub levels: [Level; 25],
}

impl Game {
    pub fn move_player(&mut self, dx: i8, dy: i8) {
        // check if move is allowed first
        let player = &self.player;
        let player_pos = player.get_position();
        let mut level = &self.levels[player_pos.get_level() as usize];
        let new_x = ((player_pos.get_x() as i16) + dx as i16) as u16;
        let new_y = ((player_pos.get_y() as i16) + dy as i16) as u16;
        let can_go: bool = match level.get_element(new_x as i16, new_y as i16) {
            None => { false }
            Some(t) => {
                match t {
                    LevelElement::Wall => { false }
                    _ => { true }
                }
            }
        };
        if can_go {
            if level.get_element(new_x as i16, new_y as i16).unwrap() == LevelElement::StairDown {
                let start_x = self.levels[(player_pos.get_level() + 1) as usize].start_x;
                let start_y = self.levels[(player_pos.get_level() + 1) as usize].start_y;
                self.player.set_position(player_pos.get_level() + 1, start_x as u16, start_y as u16);
                self.levels[(player_pos.get_level() + 1) as usize].discover(start_x as i16, start_y as i16);
            } else if level.get_element(new_x as i16, new_y as i16).unwrap() == LevelElement::StairUp {
                self.player.set_position(
                    player_pos.get_level() - 1,
                    self.levels[(player_pos.get_level() - 1) as usize].end_x as u16,
                    self.levels[(player_pos.get_level() - 1) as usize].end_y as u16,
                );
            } else {
                self.player.change_position(dx, dy);
                self.levels[player_pos.get_level() as usize].discover(new_x as i16, new_y as i16);
            }
        }
    }
}